""" Creator of views in MySQL """
import logging
import os

from django.core.exceptions import ObjectDoesNotExist
from django.db import connection

from articles.models import ParentTag, Tag, User
from cropping.models import Category

logger = logging.getLogger(__name__)


def create_view(**kwargs):
    """ Creates several MySQL-Views """
    # unused kwargs
    _ = kwargs
    logger.debug('  (re)creating partition views for Articles...')
    file_path = os.path.join(os.path.dirname(__file__), '../etc/views.sql')
    with open(file_path, encoding='UTF-8') as file:
        sql_statement = file.read()
        with connection.cursor() as cursor:
            # TODO: replace check statement with something more robust
            for statement in sql_statement.split("# ---"):
                stripped_statement = statement.replace('\n', ' ').strip()
                cursor.execute(stripped_statement)
                row = cursor.fetchone()
            logger.debug('  Done creating partition views.')
    return row


meta_tags = {
    "Allgemein": "Universität Potsdam",
    "Allgemeine Hochschultermine": "Universität Potsdam",
    "Alumni": "Universität Potsdam",
    "Anglistik und Amerikanistik": "Philosophische Fakultät",
    "Astronomie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Astrophysik": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Betriebswirtschaftslehre": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Bildungswissenschaften": "Humanwissenschaftliche Fakultät",
    "Biochemie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Biologie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Botanischer Garten": "Universität Potsdam",
    "Career Service": "Universität Potsdam",
    "Chancengleichheit": "Universität Potsdam",
    "Chemie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "DFG-Research Training Group WIPCAD": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Digital Engineering Fakultät": "Digital Engineering Fakultät",
    "Digitalisierung": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Earth Surface Dynamics": "Mathematisch-Naturwissenschaftliche Fakultät",
    "E-Learning": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Erd- und Umweltwissenschaften": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Ernährungswissenschaft": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Erwachsenenbildung": "Humanwissenschaftliche Fakultät",
    "Erziehungswissenschaft": "Humanwissenschaftliche Fakultät",
    "Ethik": "Philosophische Fakultät",
    "Fakultät für Gesundheitswissenschaften": "Fakultät für Gesundheitswissenschaften",
    "Forschung": "Universität Potsdam",
    "Forschungsbereich Komplexe Systeme": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Forschungsbereich Funktionale Weiche Materie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Forschungsbereich Politik, Verwaltung und Management": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Forschungsinitiative Netzwerk Digitale Geisteswissenschaften": "Philosophische Fakultät",
    "Forschungsschwerpunkt Data-Centric Sciences": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Forschungsschwerpunkt Erdwissenschaften/Integrierte Erdsystemanalyse": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Forschungsschwerpunkt Funktionelle Ökologie und Evolutionsforschung": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Forschungsschwerpunkt Kognitionswissenschaften": "Universität Potsdam",
    "Forschungsschwerpunkt Pflanzengenomforschung und Systembiologie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Fundraising": "Universität Potsdam",
    "Geographie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Geologie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Geoökologie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Geowissenschaften": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Germanistik": "Philosophische Fakultät",
    "Geschichte": "Philosophische Fakultät",
    "Gesundheitsmanagement": "Fakultät für Gesundheitswissenschaften",
    "Gesundheitswissenschaften": "Fakultät für Gesundheitswissenschaften",
    "Graduiertenkolleg \"Minor Cosmopolitanisms\"": "Universität Potsdam",
    "Gremientermine": "Universität Potsdam",
    "Grundschulpädagogik": "Humanwissenschaftliche Fakultät",
    "Gründung und Innovation": "Universität Potsdam",
    "Hochschuldidaktik": "Philosophische Fakultät",
    "Hochschulleitung": "Universität Potsdam",
    "Hochschulpolitik": "Universität Potsdam",
    "Hochschulsport": "Universität Potsdam",
    "Hochschulverwaltung": "Universität Potsdam",
    "Humanwissenschaftliche Fakultät": "Humanwissenschaftliche Fakultät",
    "HumboldtArt": "Universität Potsdam",
    "Informatik": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Informationsverarbeitung und Kommunikation": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Internationales": "Universität Potsdam",
    "Jüdische Studien": "Philosophische Fakultät",
    "Jüdische Theologie": "Philosophische Fakultät",
    "Juristische Fakultät": "Juristische Fakultät",
    "Klassische Philologie": "Philosophische Fakultät",
    "Klima": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Kommunikationswissenschaften": "Universität Potsdam",
    "Kulturelle Begegnungsräume": "Universität Potsdam",
    "Kulturwissenschaften": "Philosophische Fakultät",
    "Künste und Medien": "Philosophische Fakultät",
    "Lebensgestaltung-Ethik-Religionskunde (LER)": "Philosophische Fakultät",
    "Lehre": "Universität Potsdam",
    "Lehrerbildung": "Universität Potsdam",
    "Linguistik": "Humanwissenschaftliche Fakultät",
    "Mathematik": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Mathematisch-Naturwissenschaftliche Fakultät": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Musikwissenschaften": "Humanwissenschaftliche Fakultät",
    "Naturwissenschaften": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Öffentliches Recht": "Juristische Fakultät",
    "Open Science": "Universität Potsdam",
    "Partnerkreis Industrie und Wirtschaft": "Universität Potsdam",
    "Personalia": "Universität Potsdam",
    "Philosophie": "Philosophische Fakultät",
    "Philosophische Fakultät": "Philosophische Fakultät",
    "Physik": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Politik": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Politik und Verwaltung": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Politische Bildung": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Potsdam Graduate School": "Universität Potsdam",
    "Psychologie": "Humanwissenschaftliche Fakultät",
    "Qualitätsentwicklung in Lehre und Studium": "Universität Potsdam",
    "Rechtswissenschaften": "Juristische Fakultät",
    "Religionswissenschaft": "Philosophische Fakultät",
    "Romanistik": "Philosophische Fakultät",
    "Sachunterricht": "Humanwissenschaftliche Fakultät",
    "Slavistik": "Humanwissenschaftliche Fakultät",
    "Sozialwissenschaften": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Soziologie": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Sport- und Gesundheitswissenschaften": "Humanwissenschaftliche Fakultät",
    "Sprachen und Schlüsselkompetenzen": "Universität Potsdam",
    "Strafrecht": "Juristische Fakultät",
    "Studienangelegenheiten / Studium": "Universität Potsdam",
    "Studierendenvertretungen": "Universität Potsdam",
    "Systemakkreditierung": "Universität Potsdam",
    "Theodor-Fontane-Archiv": "Philosophische Fakultät",
    "Umweltwissenschaften und Geographie": "Mathematisch-Naturwissenschaftliche Fakultät",
    "Uni findet Stadt": "Universität Potsdam",
    "Universitätsbibliothek / Universitätsarchiv": "Universität Potsdam",
    "Universitätsgesellschaft e.V.": "Universität Potsdam",
    "Veranstaltungen": "Universität Potsdam",
    "Verwaltungswissenschaften": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Volkswirtschaftslehre": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Welcome Center Potsdam": "Universität Potsdam",
    "Wirtschaft-Arbeit-Technik": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Wirtschaftswissenschaften": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Weiterbildungen": "Universität Potsdam",
    "Wirtschaftsinformatik": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Wirtschafts- und Sozialwissenschaftliche Fakultät": "Wirtschafts- und Sozialwissenschaftliche Fakultät",
    "Wissenschaftlicher Nachwuchs / Nachwuchsförderung": "Universität Potsdam",
    "Wissens- und Technologietransfer": "Universität Potsdam",
    "Zivilrecht": "Juristische Fakultät"
}

pressespiegel = {
    "Universität Potsdam": "Pressespiegel",
    "Hochschulen Brandenburg": "Pressespiegel",
    "Hochschulen Berlin": "Pressespiegel",
    "IT im Hochschulbereich": "Pressespiegel",
    "Universität Potsdam in den sozialen Netzen": "Pressespiegel",
}


def connect_tags_with_parents(**kwargs):
    """ a pre defined categorization of the tags"""
    warnings = []
    for key, value in meta_tags.items():
        try:
            category = Category.objects.get(name=key)
            tag, _ = Tag.objects.update_or_create(name=category.name)
            parent_tag, _ = ParentTag.objects.update_or_create(name=value)
            if tag.parent.filter(name__contains=value).count() > 0:
                continue
            tag.parent.all().delete()
            tag.parent.add(parent_tag)
            tag.save()
        except ObjectDoesNotExist:
            warnings.append(key)

    for key, value in pressespiegel.items():
        try:
            tag = Tag.objects.get(name=key)
            if tag.parent.filter(name__contains=value).count() > 0:
                continue
            parent_tag, _ = ParentTag.objects.update_or_create(name=value)
            tag.parent.all().delete()
            tag.parent.add(parent_tag)
            tag.save()
        except ObjectDoesNotExist:
            warnings.append(key)
    # logger.warning("Tags not found: %s", ", ".join(warnings))


def create_standard_user(**kwargs):
    """ creates user with id_tag 1 """
    if User.objects.filter(id_tag=1).count() == 0:
        user = User(id_tag=1)
        user.save()
        for tag in Tag.objects.all():
            user.favorite_tags.add(tag)
        user.save()
